import 'dart:async';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_auth/jaguar_auth.dart';

import 'model.dart';

/// Model manager to authenticate against a static list of user models
class WhiteListPasswordChecker implements AuthModelManager {
  /// User models to white list
  final Map<String, User> models;

  /// Password hasher
  final Hasher hasher;

  const WhiteListPasswordChecker(Map<String, User> models, {Hasher hasher})
      : models = models ?? const {},
        hasher = hasher ?? const NoHasher();

  Future<User> authenticate(
      Context ctx, String username, String password) async {
    User model = await fetchModelByAuthenticationId(ctx, username);

    if (model == null) {
      return null;
    }

    if (!hasher.verify(password, model.loginPassword)) {
      return null;
    }

    return model;
  }

  Future<User> fetchModelByAuthenticationId(
      Context ctx, String authName) async =>
      models.values
          .firstWhere((model) => model.loginId == authName, orElse: () => null);

  Future<User> fetchModelByAuthorizationId(
      Context ctx, String sessionId) async {
    if (!models.containsKey(sessionId)) {
      return null;
    }

    return models[sessionId];
  }
}